#include "Audioh.h"

// Constructor //
// =========== //

Audioh::Audioh(bool verbose)
{
	float zeroes[3] = { 0.0f,0.0f,0.0f };
	// Initializing audio device
	this->dev=alcOpenDevice(NULL);
	if (!this->dev) throw 1;
	
	// Initializing audio context
	this->cid=alcCreateContext(this->dev,NULL);
	if (!this->cid)
	{
		alcCloseDevice(dev);
		throw 2;
	}
	alcMakeContextCurrent(this->cid);

	// Initialize buffers
	this->n_buf=0;
	this->buf = (ALuint *) malloc(1);

	// Initialize sources
	this->n_sid=0;
	this->sid = (ALuint *) malloc(1);

	this->name = (char **) malloc(1);

	// Set default listener velocity
	alListenerfv(AL_VELOCITY,zeroes);
	this->verbose=verbose;

	printf("[>] Audioh class created.\n");
}

// Destructor //
// ========== //

Audioh::~Audioh()
{
	// Free Buffers.
	if (this->buf)
	free( this->buf );

	// Free Sources
	if (this->sid)
	free (this->sid);

	// Free Device.
	if (this->dev)
	alcCloseDevice(this->dev);
	printf("[<] Audioh class destroyed.\n");
}


// Add new sound file //
// ================== //

int
Audioh::add(char *f)
{
	ALboolean err;
	ALsizei size,bits,freq;
	ALint format;
	void *wave;

	// Load Wav file

	err=alutLoadWAV(f,&wave,&format,&size,&bits,&freq);

	if (err==AL_FALSE) 
	{
		printf("   [-] - audioh - invalid file \"%s\".\n",f);
		throw 1;
	}

	this->n_buf++;
	this->buf = (ALuint *) realloc (this->buf, this->n_buf*sizeof(ALuint));
	if (!this->buf) throw 2;

	this->name = (char **) realloc(this->name,this->n_buf*sizeof(char));
	this->name[n_buf-1] = new char[20];
	strcpy(this->name[n_buf-1],f);

	alGenBuffers(1,&this->buf[this->n_buf-1]);

	alBufferData(this->buf[this->n_buf-1],
			format,wave,size,freq);

	printf("   [+] - audioh - file \"%s\" loaded OK.\n",f);
	if (verbose)
	printf("       - format: %d | freq %d | bits %d | size %d\n",
			format,freq,bits,size );

	free ( wave );

	return (this->n_buf-1);
}

// Set listener position //
// ===================== //

void
Audioh::setListener(float x,float y,float z)
{
	this->setListener(x,y,z,x,y,z);
}

void
Audioh::setListener(float x,float y,float z, float x1,float y1,float z1)
{
	float pos[6]={x,y,z,x1,y1,z1};

	alListenerfv (AL_POSITION,pos);
	alListenerfv (AL_DIRECTION,pos);
}

// Volume Manager //
// ============== //

void
Audioh::setVolume(int v)
{
	float vol;

	// Clean input.
	if (v>100) v=100; else
	if (v<0)   v=0;

	vol=(float)v/100.0f;
	
	alListenerf ( AL_GAIN,vol ); // Set new volume
}

// Play buffer on new source //
// ========================= //
//private:
bool
Audioh::checkSid(ALuint sid)
{
	if (sid > this->n_sid) // invalid pointer
		return false;
	else if (!this->sid[sid]) // destroyed SID
		return false;
	else	return true;
}

//public:
int
Audioh::sid_new(ALuint buf)
{
	unsigned int i;

	for(i=0; i < this->n_sid; i++)
		if (!this->sid[i]) break;

	if (i==this->n_sid) // Must realloc
	{
		this->n_sid++;
		this->sid = (ALuint *) 
			realloc(this->sid,this->n_sid*sizeof(ALuint));
	}

	alGenSources(1, &this->sid[i]);
	alSource3f ( this->sid[i], AL_VELOCITY, 0.0f,0.0f,0.0f );
	alSourcei  ( this->sid[i], AL_BUFFER, this->buf[buf] );

	return i;
}

void 
Audioh::sid_delete(ALuint sid)
{
	if (checkSid(sid))
	this->sid[sid]=SID_NULL;
}

void
Audioh::sid_play(ALuint sid)
{
	if (checkSid(sid))
	alSourcePlay(this->sid[sid]);
}


void
Audioh::sid_stop(ALuint sid)
{
	if (checkSid(sid))
	alSourceStop(this->sid[sid]);
}

void 
Audioh::sid_pause(ALuint sid)
{
	if (checkSid(sid))
	alSourcePlay(this->sid[sid]);
}

void 
Audioh::sid_loop(ALuint sid, bool loop)
{
	if (checkSid(sid))
	alSourcei (this->sid[sid], AL_LOOPING, (int)((loop)?1:0));
}

void
Audioh::sid_pos(ALuint sid,
		float x,float y, float z)
{
	this->sid_pos(sid,x,y,z,x,y,z);
}

void
Audioh::sid_pos(ALuint sid,
		float x, float y, float z,
		float x1, float y1, float z1)
{
	float ori[6]={ x,y,z,x1,y1,z1 };

	if (checkSid(sid))
	{
	alSource3f(this->sid[sid], AL_POSITION, x,y,z);
	alSourcefv(this->sid[sid], AL_ORIENTATION, ori );
	}
}

void
Audioh::sid_cone(ALuint sid, int angle, float in, float out)
{
	if (checkSid(sid))
	{
	alSourcei(this->sid[sid],AL_CONE_OUTER_ANGLE, angle);
	alSourcef(this->sid[sid],AL_CONE_OUTER_ANGLE, out);
	alSourcef(this->sid[sid],AL_CONE_INNER_ANGLE, in);
	}
}

// Get Info about SiDs //
// =================== //

ALint Audioh::sid_state(ALuint sid)
{
	ALint state;

	if (sid==SID_NULL||sid>this->n_sid) return 0;
	if (alIsSource(this->sid[sid])==AL_FALSE)
		return AL_FALSE;
	state=AL_INITIAL;
	alGetSourceiv(this->sid[sid],AL_SOURCE_STATE,&state);

	// #include <AL/altypes.h>
	// state could be: AL_PLAYING, AL_PAUSED, AL_STOPPED
	// or other not enought interesting.
	return state;
}

void Audioh::stopAll(){
	unsigned int i=0;
	for (i=0;i<n_sid;i++){
		if (sid_state(i)==AL_PLAYING)  sid_stop(i);
	}
}

int Audioh::getNumBuffers(){
	return n_buf;
}

int Audioh::getNumSources(){
	return n_sid;
}

char *Audioh::getFileName(ALuint sound){
	if (sound>=0 && sound <n_buf) return name[sound];
	else return "NO NAME";
}



