#ifndef _AUDIO_H_
#define _AUDIO_H_

#include <stdio.h>
#include <string.h>
#include <AL/al.h>
#include <AL/alc.h>
#include <AL/alut.h>
#include <limits.h>
#define SID_NULL UINT_MAX

#include <time.h>
#include <stdlib.h>


// Audioh class definition //
// ======================= //

#ifdef WIN32
#define alGetSourceiv alGetSourcei


inline ALboolean alutLoadWAV(const char *fname,ALvoid **buffer,ALint *format,
       ALsizei *size,ALsizei *bits,ALsizei *freq) 
{
	ALboolean loop;
	alutLoadWAVFile(const_cast<char *>(fname),format,buffer,size,freq,&loop);
	if(*format==AL_FORMAT_STEREO16 || *format==AL_FORMAT_MONO16) 
		*bits=16;
	else  *bits=8;
	return AL_TRUE;
}

#endif

class Audioh
{
	private:
	ALCdevice *dev;        // Audio device
	ALCcontext *cid;        // Context ID
	bool verbose;          // Verbose mode?
	ALuint *buf,     // Audio Buffers
	       *sid;     // Audio Sources
	unsigned int n_buf, n_sid;

	char **name;

	bool checkSid(ALuint sid);
	
	public:
	Audioh(bool verbose=false); // Ctor
	~Audioh();                  // Dtor

	// Add/remove data
	int add(char *f);        // Add new audio file. returns ID.

	// General Settings
	void setListener(float x, float y, float z); // Position.
	void setListener(float x, float y, float z,   // Position
			 float x1, float y1, float z1 ); // Orientation
	void setVolume(int v);      // Set new volume

	// Sources
	int  sid_new   (ALuint buf);
	void sid_delete(ALuint buf);
	void sid_play  (ALuint sid);
	void sid_stop  (ALuint sid);
	void sid_pause (ALuint sid);
	void sid_cone  (ALuint sid, int angle, float in, float out); 
	void sid_loop  (ALuint sid, bool loop);
	void sid_pos   (ALuint sid, float x, float y, float z); // Position
	void sid_pos   (ALuint sid, float x, float y, float z,  // Position
			            float x1, float y1, float z1); // Orientation
	int  sid_state (ALuint sid);

	void stopAll();
	int getNumBuffers();
	int getNumSources();
	char *getFileName(ALuint sound);
};

#endif

