#ifndef _GLIMAGE_H_
#define _GLIMAGE_H_

#if WIN32
#include <windows.h>
#endif
#include <GL/glu.h>
#include <GL/gl.h>

/****************************************************************************
*
*	CLASSNAME:		GLImage		 
*
*	DESCRIPTION:	Imatge. Vector de dades d'imatge. 
*
****************************************************************************/
/****************************************************************************
*   <table border=2 >
*	<tr><td><b>Version	</b></td><td> 1.0				</td></tr>
*	<tr><td><b>Author	</b></td><td> Citem Simulation Grup	</td></tr> 
*	<tr><td><b>Data		</b></td><td> -- / -- / 2001	</td></tr>
*	<tr><td><b>e-mail	</b></td><td><a href="mailto:tm06521@salleurl.edu"> 
*									  tm06521@salleUrl.edu
*									 </a></td></tr>
*	<tr><td><b>To do	</b></td><td>					</td></tr>
*	</table>
*****************************************************************************/

class GLImage  
{
private:
	
		
	// tipo de cada component siempre  GL_UNSIGNED_BYTE
	GLenum  m_type;		
	// Amplada
	GLsizei	m_width;	
	// alada
	GLsizei	m_height;	
public:
	// vector de dades
	GLubyte *m_data;
	// format GL_RGB o GL_RGBA solamente
	GLenum  m_format;

public:
	// Constructor inicialitza als valors per defecte
	GLImage();
	// Destructor allibera la memoria del vector "m_data"
	virtual ~GLImage();

	// Inicialitza una imatge.
	void set(
				GLsizei width,	// Amplada 
				GLsizei height, // Alada
				GLenum format,	// Format GL_RGB o GL_RGBA
				GLenum type,	// Tipus sempre GL_UNSIGNED_BYTE
				GLubyte* data	// vector de dades.
			);

	// Defineix el format 
	void setFormat( GLenum format // Format GL_RGB o GL_RGBA
				  );
	// Defineix el Typus de dades 
	void setType(GLenum type	// Tipus sempre GL_UNSIGNED_BYTE
				);

	// Retorna l'amplada de la imatge
	GLsizei  getWidth();
	// Retorna l'alada de la imatge
	GLsizei  getHeight();
	// Retorna el format
	GLenum   getFormat();
	// Retorna el Tipus
	GLenum   getType();
	// Retorna el punter al vector de dades.
	GLubyte* getData();

	// Sobre carrega del oprador =. Copiem la imatge.
	void operator=(const GLImage &image);

	// Crea la imatge a partir d'un lectura del Buffer.
	bool createFromBuffer( int x,		// Posicio x a partir de la qual copiarem el Buffer
		                   int y,		// Posicio y a partir de la qual copiarem el Buffer
						   int width,	// Amplada a partir del tro de Buffer a copiar
						   int height   // Alada a partir del tro de Buffer a copiar
						 );
	// Dibuixa la imatge.
	void draw(	GLfloat posX,	// posicio X del punt inferior esquerra de la imatge
				GLfloat posY,	// posicio Y del punt inferior esquerra de la imatge
				GLfloat posZ	// posicio Z del punt inferior esquerra de la imatge
			 );

	// Redimenciona una imatge.
	bool resize( GLsizei width, // Nova amplada
				 GLsizei height	// Nova Alada
			   );
	// Esborra la imatge. ( Borrar tot el vector de Dades.
	void clear();

	// Copia la mitga dels color RGB al canal alpha.
	bool setAlphaEqualRGB();

	// Zoom a per dibuixa la imatge.
	static void drawZoom(GLfloat zoomX, GLfloat zoomY);

};

#endif 
