// GLTGAParser.cpp: implementation of the GLTGAParser class.
//
//////////////////////////////////////////////////////////////////////

//#include "stdafx.h"
#include "GLTGAParser.h"



GLenum GLTGAParser::format;
int GLTGAParser::size;
int GLTGAParser::bits;
int GLTGAParser::height;
int GLTGAParser::width;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GLTGAParser::GLTGAParser()
{

}

GLTGAParser::~GLTGAParser()
{

}

//////////////////////////////////////////////////////////////////////
// Member function
//////////////////////////////////////////////////////////////////////
bool GLTGAParser::load(char *fileName, GLImage &img)
{

	/****** Codi del TGATEXTURE ************/
	byte type[4];
	byte info[7];

	byte* data;

	FILE* iFile;

	if (!(iFile = fopen(fileName, "r+bt")))
		//return getError(notFound, 0);
		return false;

	fread(&type, sizeof(byte), 3, iFile); // read in colormap info and image type, byte 0 ignored
	fseek(iFile, 12, SEEK_SET); // seek past the header and useless info
	fread(&info, sizeof(byte), 6, iFile);
	fseek(iFile, type[0], SEEK_CUR); // skip past image identification

	if (type[1] != 0 || (type[2] != 2 && type[2] != 3))
		//return getError (badType, iFile);
		return false;

	width = info[0] + info[1] * 256; 
	height = info[2] + info[3] * 256;
	bits =	info[4]; 

	size = width * height; 

   // make sure dimension is a power of 2 
   //if (!CheckSize (width) || !CheckSize (height))
      //return Error(badDimension, iFile);

   // make sure we are loading a supported type 
	if (bits != 32 && bits != 24 && bits != 8)
		//return getError(badBits, iFile);
		return false;

	data = loadData(iFile, size);

	// no image data 
	if (data == 0)
		//return getError(badData, iFile);
		return false;

	fclose(iFile);

   	/****** Fi codi del TGATEXTURE *********/

	img.set(width, height, format, GL_UNSIGNED_BYTE, data);

	return true;
}

bool GLTGAParser::writeToFile(char *fileName, GLImage &img)
{
	short int width = (short) img.getWidth();
	short int height = (short) img.getHeight();
	GLenum format = img.getFormat();
	unsigned char pixelDepth = 24;
	if ( format == GL_RGBA ) pixelDepth = 32;

	unsigned char* imageData;
	
	unsigned char cGarbage = 0, type,mode,aux;
	short int iGarbage = 0;
	int i;
	
	
	mode = pixelDepth / 8;

	imageData = new unsigned char[width * height * mode];

	for(i = 0 ; i < width * height * mode ; i++)
	{
		imageData[i] = img.getData()[i];
	}

	FILE *file;

// open file and check for errors
	file = fopen(fileName, "wb");
	if (file == NULL) {
		return false;
	}

// compute image type: 2 for RGB(A), 3 for greyscale
	
	if ((pixelDepth == 24) || (pixelDepth == 32))
		type = 2;
	else
		type = 3;

// write the header
	fwrite(&cGarbage, sizeof(unsigned char), 1, file);
	fwrite(&cGarbage, sizeof(unsigned char), 1, file);

	fwrite(&type, sizeof(unsigned char), 1, file);

	fwrite(&iGarbage, sizeof(short int), 1, file);
	fwrite(&iGarbage, sizeof(short int), 1, file);
	fwrite(&cGarbage, sizeof(unsigned char), 1, file);
	fwrite(&iGarbage, sizeof(short int), 1, file);
	fwrite(&iGarbage, sizeof(short int), 1, file);

	fwrite(&width, sizeof(short int), 1, file);
	fwrite(&height, sizeof(short int), 1, file);
	fwrite(&pixelDepth, sizeof(unsigned char), 1, file);

	fwrite(&cGarbage, sizeof(unsigned char), 1, file);

// convert the image data from RGB(a) to BGR(A)
	if (mode >= 3)
	for (i=0; i < width * height * mode ; i+= mode) {
		aux = imageData[i];
		imageData[i] = imageData[i+2];
		imageData[i+2] = aux;
	}

// save the image data
	fwrite(imageData, sizeof(unsigned char), 
			width * height * mode, file);
	/*for ( i = 0; i < width * height * mode ; i+=width*4 ) { 
		fwrite(&imageData[width * height * mode - i], sizeof(unsigned char), width*4, file);
	}*/

	fclose(file);
// release the memory
	free(imageData);

	return true;
}

byte* GLTGAParser::loadRGBA( FILE* strm, int size ) {
   byte *rgba;
   byte temp;
   int bread;
   int i;

   rgba = new byte[size * 4]; 

   if (rgba == 0)
      return 0;

   bread = fread(rgba, sizeof(byte), size * 4, strm); 

   if (bread != size * 4)
   {
      delete [] rgba;
      return 0;
   }

   for (i = 0; i < size * 4; i += 4 )
   {
		temp = rgba[i];
		rgba[i] = rgba[i + 2];
		rgba[i + 2] = temp;
   }

   format = GL_RGBA;
   
   return rgba;
}

byte* GLTGAParser::loadRGB( FILE* strm, int size ) {
   byte *rgb;
   byte temp;
   int bread;
   int i;

   rgb = new byte[size * 3]; 

   if (rgb == 0)
      return 0;

   bread = fread (rgb, sizeof(byte), size * 3, strm);

   if (bread != size * 3)
   {
      delete [] rgb;
      return 0;
   }

   for (i = 0; i < size * 3; i += 3)
   {
      temp = rgb[i];
      rgb[i] = rgb[i + 2];
      rgb[i + 2] = temp;
   }

   format = GL_RGB;

   return rgb;
}

byte* GLTGAParser::loadGray( FILE *strm, int size ) {
   byte *grayData;
   int bread;

   grayData = new byte[size];

   if (grayData == 0)
      return 0;

   bread = fread(grayData, sizeof(byte), size, strm);

   if (bread != size)
   {
      delete [] grayData;
      return 0;
   }

   format = GL_ALPHA;

   return grayData;
}

byte* GLTGAParser::loadData( FILE* strm, int size ) {
   if (bits == 32)
      return loadRGBA (strm, size);
   else if (bits == 24)
      return loadRGB (strm, size);	
   else if (bits == 8)
      return loadGray (strm, size);
   
   return 0;
}
