#include "Object3D.h"


object3d::object3d(char *file)
{
	animacioTotal=0;
	numFigures=-1;
	texturaObj=texture();

	//Cargamos el modelo
	if (!this->leeArchivo(file))
	{
		printf("\n##### MODEL %s NOT FOUND #####\n\n",file);
		killProgram(0);
	}
}

object3d::~object3d(){
	delete[] textureVertex;
	
	for (int i=0;i<numFigures;i++)
		delete figura[i];
	delete[] figura;

	delete[] animacio;
}

int object3d::leeArchivo(char *file){
	FILE* fd;
	char buf[100];
	int k,animPos=0,conta=-1;

	float *vertexTempo1=0,*vertexTempo2=0;
	float *vertexTextureTempo=0;
	int *faceTempo=0;
	int *faceMapTempo=0;
 	int numVertex;
	GL3f vertex;
	GL3i facestemp;
	GL2f textureP;
	
	if (! chdir(getDirs(file)) )
	{
		fd=fopen (file,"r");
		chdir("../..");
	}

	if (!fd) 
	{
		system("pwd");
		printf("[e] Object3D::Cannot open file %s\n",file);
		return 0;
	}
		
	//Leemos la cabecera del modelo
	do{
		buf[0]=0;
		fscanf(fd,"%s",buf);
		if (!strcmp(buf,"&NUM_FRAMES")){
			fscanf(fd,"%d",&numFigures);
			figura=new figure*[numFigures];
		}
		else if (!strcmp(buf,"&NUM_ANIMATIONS")){
			fscanf(fd,"%d",&animacioTotal);
			animacio=new int[(animacioTotal*2)+1];
		}
		else if (!strcmp(buf,"&TEXTURE")) {
			fscanf(fd,"%s",buf);
			addTexture(buf);							
		}
		else if (!strcmp(buf,"&SPEED")){
			fscanf(fd,"%d",&speedDefault);
		}
		else if (!strcmp(buf,"&MAP_PARTS")){
			fscanf(fd,"%d",&numTextureVertex);

			//Este array servira posteriormente para colocar los vertices de textura del
			//vertexArray. Multiplicamos el numero de caras por 6 ya que cada cara tiene
			//3 vertices y cada vertice 2 componentes (u,v)
			textureVertex=new float[numFaces*6];					
			vertexTextureTempo=new float[numTextureVertex*2];
			for (k=0;k<numTextureVertex*2;k+=2){
				fscanf(fd,"%f %f ",&vertexTextureTempo[k], &vertexTextureTempo[k+1]);
				vertexTextureTempo[k+1]=1.0f-vertexTextureTempo[k+1];
			}
		}
		else if (!strcmp(buf,"&MAP_FACES")){
			fscanf(fd,"%d",&numFaces);
			faceMapTempo=new int[numFaces*3];
			for (k=0;k<numFaces*3;k+=3){
				fscanf(fd,"%d %d %d\n",&faceMapTempo[k],&faceMapTempo[k+1],&faceMapTempo[k+2]);
			}
		}
		else if (!strcmp(buf,"&FACES")){
			fscanf(fd,"%d",&numFaces);
			faceTempo=new int[numFaces*3];				
			for (k=0;k<numFaces*3;k+=3){
				fscanf(fd,"%d %d %d\n",&faceTempo[k],&faceTempo[k+1],&faceTempo[k+2]);
			}
		}
		else if (!strcmp(buf,"&VERTEX")){
			fscanf(fd,"%d",&numVertex);
		}
	} while(buf[0]!='*');

	vertexTempo1=new float[numVertex*3];

	//Este array servira posteriormente para colocar los vertices del
	//vertexArray. Multiplicamos el numero de caras por 9 ya que cada cara tiene
	//3 vertices y cada vertice 3 componentes (x,y,z)
	vertexTempo2=new float[numFaces*9];
		
	//Leemos las animaciones
	for (register int cont=0;cont<numFigures && !feof(fd);cont++){
		
		//Comprobamos si es una nueva animacion o una continuacion de otra
		if (buf[0]=='*')
			{

			//Guardamos que frame es el primero de esta animacion
			animacio[animPos]=cont;

			//Miramos de que tipo es y tambien lo guardamos
			if (buf[1]=='S')
				{
				animacio[animPos+1]=1;
				} 
			else if(buf[1]=='D')
				{
				animacio[animPos+1]=0;
				}
			animPos+=2;
			}
		else if (buf[0]!='.')
			{
			return 0;
			}

		//Creamos un nuevo objeto figure y le pasamos el numero de caras del modelo
		figura[cont] = new figure(numFaces);
		

		//Leemos los vertices
		for (k=0;k<numVertex*3;k+=3)
			{
			
			fscanf(fd,"%f %f %f",&vertexTempo1[k],&vertexTempo1[k+1],&vertexTempo1[k+2]);
			vertexTempo1[k]*=RATIO;
			vertexTempo1[k+1]*=RATIO;
			vertexTempo1[k+2]*=RATIO;
			}
		

		//Ponemos los vertices ordenados por caras en el array correspondiente al vertexArray
		//Miramos uno por uno cada vertice de cada cara y ponemos sus 3 componentes en el
		//vertexArray
		for (k=0;k<numFaces*3;k++)
			{
			vertexTempo2[(k*3)]=vertexTempo1[faceTempo[k]*3];
			vertexTempo2[(k*3)+1]=vertexTempo1[(faceTempo[k]*3)+1];
			vertexTempo2[(k*3)+2]=vertexTempo1[(faceTempo[k]*3)+2];
			}
		
		//Le pasamos los vertices al objeto figura correspondiente
		figura[cont]->addVertex(vertexTempo2);

		//Volvemos a leer el fichero en busca de un * o un .
		fscanf(fd,"%s",buf);
	}

	//Cerramos el fd
	fclose(fd);

	//Creamos su bounding con los vertices del primer frame
	bounding=GLBounding(figura[0]->getVertex(),numFaces*3);


	//Ponemos los vertices de textura ordenados por caras en el array correspondiente 
	//al vertexArray de texturas.
	//Miramos uno por uno cada vertice de textura de cada cara y ponemos sus 2 componentes en el
	//vertexArray de texturas
	for (k=0;k<numFaces*3;k++){
		textureVertex[(k*2)]=vertexTextureTempo[faceMapTempo[k]*2];
		textureVertex[(k*2)+1]=vertexTextureTempo[(faceMapTempo[k]*2)+1];
	}
			
	//Creamos una animacion ficticia que empiece en el frame siguiente al ultimo.
	//Servira para saber cuando acaba la ultima animacion
	animacio[animPos]=numFigures;

	//Liberamos memoria
	delete vertexTextureTempo;
	delete vertexTempo1;
	delete vertexTempo2;
	
	//Todo fue bien
	return 1;
	
};

int object3d::addTexture(char *textura_old){


	//Variables temporales
	char *texturaTemp;
	texturaTemp=new char[100];

	//Inicializamos la textura
	texturaObj=texture();
	texturaObj.setName(textura_old);
	glEnable(GL_TEXTURE_2D);
	//Cargo la textura en memoria
	try{
	  texturaTemp=TGAload(textura_old,&texturaObj.width,&texturaObj.height,&texturaObj.bpp);
	}catch(int err){
		printf("ERROR %d LOADING TEXTURE\n",err);
		return 0;
	}


	//Compruebo que se ha cargado bien
	if (!texturaTemp) return 0;
	

	//Inicalizo los parametros correspondientes a Opengl
	glGenTextures(1,&texturaObj.idTex);
	glBindTexture(GL_TEXTURE_2D, texturaObj.idTex);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	if (texturaObj.bpp==32)
	  gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,texturaObj.width,texturaObj.height,GL_RGBA,GL_UNSIGNED_BYTE,texturaTemp);
    else if(texturaObj.bpp==24)
      gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,texturaObj.width,texturaObj.height,GL_RGB,GL_UNSIGNED_BYTE,texturaTemp);
	else {
		printf ("Texture %s loaded incorrectly\n",textura_old);
		return 1;
	}
		
	
	//Liberamos memoria
//	free (texturaTemp);

	//Todo fue bien
	return 1;
};


int object3d::getNumFigures(){ 
	return numFigures;
};

int object3d::getAnimacioTotal(){
	return animacioTotal;
};

 figure **object3d::getFigura(){
	return figura; 
};

 float *object3d::getTextureVertex(){
	return textureVertex;
};

 float *object3d::getVertex(int posAnimacio){
	return figura[posAnimacio]->getVertex();
};

int object3d::getSpeedDefault(){
	return speedDefault;
};


texture object3d::getTexture(){
	return texturaObj;
}

GLBounding object3d::getBounding(){
	return bounding;
}

void object3d::setBounding(GLBounding bounding_old){
	bounding=bounding_old;
}

int object3d::getTipoAnimacio(int num){
	return animacio[(num*2)+1];
}

int object3d::getFirstFrame(int num){
	return animacio[num*2];
}

int object3d::getNumTextureVertex(){
	return numTextureVertex;
}

int object3d::getNumFaces(){
	return numFaces;
};

