#ifndef _OBJECT3D_H_
#define _OBJECT3D_H_

#include "figure.h"
#include "help.h"
#include "define.h"
#include <string.h>
#include "dirs.h"
#include "TGAload.h"

extern void killProgram(int mode);

//Controla la geometria, texturas, etc. de los objetos mobiles
class object3d{

private:
	float *textureVertex;	        //Array con los vertices de textura
	int numTextureVertex;           //Numero de texturas de vertice
	texture texturaObj;             //La textura del modelo
	figure **figura;		//Array donde se guardan los diferentes frames de animacion 
					//de los modelos 
	int numFigures;			//Numero de frames que tiene el modelo
	int *animacio;			//Array donde se guarda que frame es el primero de cada animacin
					//y despues un 0 si es dinamica o un 1 si es estatica
	int speedDefault;		//Define la velocidad por defecto
	GLBounding bounding;            //El bounding del objeto
	int animacioTotal;		//Define el total de animaciones que tiene el modelo
	int numFaces;			//Numero de caras del objeto

	//Funcion que lee el archivo
	int leeArchivo(char *ruta);

	//Aade y inicializa la textura
	//Devuelve 1 si tod fue bien y 0 en caso contrario
	int addTexture(char *textura);

	//Le asignas un bounding al objeto.
	//Espera una estructura GLBounding
	void setBounding(GLBounding bounding_old);
	
public:
	
	//Constructor por defecto (inicializa variables)
	object3d(){}

	//Constructor que espera la ruta a un archivo y lo carga
	object3d(char *ruta);

	//Destructor
	~object3d();
	
	//Devuelve el total de animaciones que hay
	int getAnimacioTotal();

	//Retorna el numero de frames que tiene el modelo
	int getNumFigures();

	//Devuelve el array de frames
	figure **getFigura();
	
	//Devuelve la posicin de los 2 vertices de las texturas que tu le pidas
    float *getTextureVertex();

	//Devuelve el numero de texturas de vertice q tiene el objeto
	int getNumTextureVertex();

	//Devuelve los vertices de un frame determinado
    float *getVertex(int posAnimacio);
		
	//Devuelve la velocidad por defecto
	int getSpeedDefault();

	//Devuelve la textura del objeto
	texture getTexture();

	//Devuelve el bounding del objeto
	GLBounding getBounding();

	//Devuelve 1 si la animacion es dinamica y 0 si es estatica
	//Espera el numero de animacion q qeremos saber
	int getTipoAnimacio(int num);

	//Devuelve el primer frame de la animacion q le pidamos
	int getFirstFrame(int num);

	//Devuelve el numero de caras que tiene el objeto
	int getNumFaces();

};

#endif


