#ifndef _CONTROL_H_
#define _CONTROL_H_

#include "Object3D.h"
#include <SDL/SDL.h>
#include "mates.h"


//Clase q controla la informacion de los modelos (velocidad, posicion, rotacion, etc.)
class control {
private:	
	object3d *object3dObj;  //Aqui se guarda la informacion 3d del objeto
	bool object3dPointer;   //Es verdadero si el object3dObj es un puntero a
							//un object3d externo a la clase.
	GL3f position;			//Posicion actual del modelo
	float rotationAngle;	//Cantidad de rotacion del objeto
	int rotationSpeed;	    //Cada cuanto gira el objeto (si esta activado)
	float rotation;			//Rotacion actual del modelo
	unsigned int  final;	//Momento en el que cambia de animacion
	unsigned int  rotateTimeLeft;
		
	int numAnimacio;		//Define la animacion en la que estas actualmente
	int nextAnimacio;		//La siguiente animacin que habr
	int posAnimacio;		//Define en que frame estas actualmente
	int speed;				//Define la velocidad actual del modelo
	bool visible;			//Define si un objeto es visible o no
#if _PVS
	int id;					//Indica la habitacion en la que esta el objeto
#endif
	int desintegracion;	    //Si es positivo el objeto aparece, si es negativo desaparece
							//y si vale 0 no pasa nada
	unsigned int nextDesintegration;
	int numDFaces;
	unsigned int nextExplosion;
	float *explodeVertex;
	float explosionRang;
	bool explode;
	
	void inicializaVariables();
public:

	//Constructor por defecto que recibe la ruta al modelo
	control(char *ruta);

	//Constructor que recibe la ruta al modelo y su posicion
	control(char *ruta, float positionx, float positiony, float positionz);

	//Constructor por defecto que recibe un puntero al model
	control(object3d *objeto_viejo);

	//Constructor que recibe un puntero al modelo y su posicion
	control(object3d *objeto_viejo, float positionx, float positiony, float positionz);

	//Destructor
	~control();

	//Constructor de copia
	control(const control &control_);

	//Funcion que controla el tiempo y las animaciones
	void update();

	//Funcion para cambiar de animacion que devuelve 0 si la animacion no existe
	int setAnimation(int numAnimacio_x, int inmediate=0, bool changeAnyWay=false);

	//Devuelve la animacion actual
	int getNumAnimacio();

        //Devuelve el frame de la animacion actual
	int getPosAnimacio();


	//Devuelve la velocidad actual
	int getSpeed();

	//Modifica la velocidad entre 20 y 1500
	void setSpeed(int speed_copia);


	//Funcion que retorna la posicion actual del objeto
	GL3f getPos();

        //Le asignamos una posicion al objeto
	void setPos(GL3f pos);

	//Funcion que retorna la rotacion del objeto
	float getRotation();

        //Devuelve el angulo de rotacion del objeto
	float getRotationAngle();

	//Modifica la posicion del objeto
	void setPos(float posX, float posY, float posZ);

	//Modifica la rotacion del objeto
	void setRotation(float rotation_old);

	//Modifica el angulo de rotacion del objeto
	void setRotationAngle (float rotationAngle_old);

        //Activas la rotacion automatica, le has de pasar la velocidad de rotacion
	void activateRotation(int rotationSpeed_old);

        //Le pasas un nuevo objeto3d
	void setObject3d(object3d *objeto_viejo);

        //Devuelve el objeto object3d actual
	object3d *getObject3d();

	//Activa o desactiva la invisibilidad
	void setVisible(bool mode);

	//Devuelve true si el objeto es visible o false si no
	bool isVisible();

#if _PVS
	//Permite modificar el id de la habitacion en la que estas
	void setId(int id_);

	//Devuelve el id de la habitacion en la que estas
	int getId();
#endif

	//Activa o desactiva la desintegracion
	void setDesintegration(int mode);

	//Devuelve si el objeto se esta desintegrando o no
	int isDesintegrating();

	//Devuelve el numero de vertices del objeto desintegrados
	int getNumDFaces();

	void setExplosion(bool mode);

	bool isExploting();

	float *getExplotingVertex();
};

#endif
