// fuentes.c	Cosas relativas a las fuentes....



#include "fuentes.h"


texto2D::texto2D(){

}

texto2D::texto2D(char *fichero){

	if (!carga_fuente(fichero)) printf("ERROR CREATING FONT %s\n",fichero);
}

/*! Se crea una textura y un mapa de coordenadas de 128*4 elementos donde se almacenan 
las coordenadas para cada carcter. El mapa de fuentes es un TGA de 256x256 con canal ALPHA.*/

bool texto2D::carga_fuente(char *fichero) {
	

	char *textura_datos;	/* Temporal; para guardar el bitmap */

	int width,height,bpp;

	
		
	/* Fuentes texturizadas */
	glGenTextures (1, &mapabits);	

	try{
	  textura_datos=TGAload(fichero,&width,&height,&bpp);
	}catch(int err){
		printf("ERROR %d LOADING TEXTURE\n",err);
		return false;
	}
	if ((textura_datos == NULL))  {
        return false;
    }

	 
	glGenTextures(1,&mapabits);
	glBindTexture (GL_TEXTURE_2D, mapabits);

	
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_NEAREST);

	if (bpp==32)
	  gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,width,height,GL_RGBA,GL_UNSIGNED_BYTE,textura_datos);
	else if (bpp==24)
	  gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,width,height,GL_RGB,GL_UNSIGNED_BYTE,textura_datos);
	else {
		printf("WRONG NUMBER OF BPP\n");
		return false;
	}

	free (textura_datos);  

	return true;
}

/*! Imprime un texto con la fuente que se le pasa. Como vemos, esta funcin
admite un nmero indeterminado de parmetros que se parsean al principio. Los 
ltimos parametros son los de formato de cadenas, como el printf*/


void texto2D::print(char *texto,float x,float y,float r,float g,float b,float size){

	int i;
	
	y=480-y;
	y=y*HEIGHT/480;
	x=x*WIDTH/640;
	size=size*WIDTH/640;
	glBindTexture (GL_TEXTURE_2D, mapabits);
	

	glPolygonMode(GL_FRONT, GL_FILL);
	if (TRANSPARENCIA){
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER,0.9);
	}
	
	if (!glIsEnabled(GL_TEXTURE_2D)) glEnable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
	glColor3f(r,g,b);

	for (i=0;i<(int)strlen(texto);i++) {
		char c=texto[i];
		
		
		int numChar;
		if (c>='0' && c<='9') numChar=(int)c-48;
		else if(c>='A' && c<='Z') numChar=(int)c-55;
		else if(c>='a' && c<='z') numChar=(int)c-87;
		else if(c=='!') numChar=36;
		else if(c=='(' || c=='{') numChar=37;
		else if(c==')' || c=='}') numChar=38;
		else if(c=='?' || c=='') numChar=39;
		else if(c==':') numChar=40;
		else if(c==';') numChar=41;
		else if(c=='-') numChar=42;
		else if(c=='.') numChar=43;
		else if(c=='*') numChar=44;
		else if(c=='_') numChar=45;
		else numChar=-1;

		if (numChar>=0){
			int row=numChar/8;
			int col=numChar%8;

			float ox,oy,ex,ey;

			ox=(col)*(1.0f/8.0f);
			ex=(col+1)*(1.0f/8.0f);
			oy=1-(row)*(1.0f/8.0f);
			ey=1-(row+1)*(1.0f/8.0f);

			glTexCoord2f(ox,ey);
			glVertex3f(x+(i*size),y,0);

			glTexCoord2f(ex,ey);
			glVertex3f(x+(i+1)*size,y,0);

			glTexCoord2f(ex,oy);
			glVertex3f(x+(i+1)*size,y+size,0);

			glTexCoord2f(ox,oy);
			glVertex3f(x+i*size,y+size,0);
		}
		
	}
	
	glEnd();

	if (TRANSPARENCIA) glDisable(GL_ALPHA_TEST);

}	

void texto2D::translate(int num, char* palabras){
	if (num>=0 && num<=9) {
		palabras[0]=num+48;
		palabras[1]=0;
	}
	else if (num>=10 && num<=35){
		palabras[0]=num+55;
		palabras[1]=num+87;
		palabras[2]=0;
	}
	else if (num==36) {
		palabras[0]='!';	
		palabras[1]=0;
	}
	else if (num==37) {
		palabras[0]='(';	
		palabras[1]='{';
		palabras[2]=0;
	}
	else if (num==38) {
		palabras[0]=')';	
		palabras[1]='}';	
		palabras[2]=0;
	}
	else if (num==39) {
		palabras[0]='?';	
		palabras[1]='';	
		palabras[2]=0;
	}
	else if (num==40) {
		palabras[0]=':';	
		palabras[1]=0;
	}
	else if (num==41) {
		palabras[0]=';';	
		palabras[1]=0;
	}
	else if (num==42) {
		palabras[0]='-';	
		palabras[1]=0;
	}
	else if (num==43) {
		palabras[0]='.';	
		palabras[1]=0;
	}
	else if (num==44) {
		palabras[0]='*';	
		palabras[1]=0;
	}
	else if (num==45) {
		palabras[0]='_';	
		palabras[1]=0;
	}
	else palabras[0]=0;
	

}
			
