#include "map.h"

map::map(){
	numHabitaciones=0;
	numTextures=0;
	textureArray=0;
	habitacionObj=0;
	try{
		sound=new Audioh();
	}catch(int err){
		printf("err %d loading sound\n",err);
	}
	soundRef=(int *)malloc(1);
	sound->setVolume(SOUND_VOLUME);
}

map::~map(){

	delete[] textureArray;

	for (int i=0;i<numHabitaciones;i++){
		delete habitacionObj[i];
	}
	delete[] habitacionObj;

	printf("MAP DESTROYED\n");
}

int map::getNumHabitaciones(){
	return numHabitaciones;
};

habitacion *map::getHabitacion(int pos){ 
	if (pos>=0 && pos<numHabitaciones) return habitacionObj[pos];
	else return 0;
}

int map::addTexture(char *textura,int num){

	//Variable temporal
	char *texturaTemp;
	texturaTemp=new char[100];

	//Comprobamos si la posicion de la textura es valida
	if (num<0 || num>=numTextures) return 0;

	//Inicializamos la textura
	textureArray[num]=texture();
	textureArray[num].setName(textura);

	try{
		//Cargamos la textura
		texturaTemp=TGAload(textura,&textureArray[num].width,&textureArray[num].height,&textureArray[num].bpp);
	}
	catch(int err){
		printf("ERROR %d LOADING TEXTURE %s\n",err,textura);
	}

	//Comprobamos si la textura se ha cargado correctamente
	if (!texturaTemp) return 0;

	//Inicializamos los parametros de opengl correspondientes a la textura
	glGenTextures(1,&textureArray[num].idTex);
	glBindTexture(GL_TEXTURE_2D, textureArray[num].idTex);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	if(textureArray[num].bpp==32)
	  gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,textureArray[num].width,textureArray[num].height,GL_RGBA,GL_UNSIGNED_BYTE,texturaTemp);
	else if(textureArray[num].bpp==24)
	  gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,textureArray[num].width,textureArray[num].height,GL_RGB,GL_UNSIGNED_BYTE,texturaTemp);
	else {
		printf("ERROR LOADING TEXTURE\n");
		return 0;
	}

	//Liberamos memoria
	delete[] texturaTemp;

	//Todo ok!!!
	return 1;
}

texture map::getTexture(int pos){
	if (pos>=0 && pos<numTextures) return textureArray[pos];
	return texture();
}
	
int map::createHabitaciones(int num){
	if (num<=0) return 0;
	habitacionObj=new habitacion*[num];
	if (!habitacionObj) return 0;
	numHabitaciones=num;
	printf("%d habitaciones creadas\n",numHabitaciones);	
	return 1;
}

int map::createTextures(int num){
	if (num<=0) return 0;
	textureArray=new texture[num];
//	if (!habitacionObj) return 0;
	numTextures=num;
	return 1;
}

int map::addHabitacion(int id){
	if (id<0 || id>=numHabitaciones) return 0;
	habitacionObj[id]=new habitacion(id);
	if (!habitacionObj[id]) return 0;

	return 1;
}

int map::getNumTextures(){
	return numTextures;
}

Sky map::getSky(){
	return fondo;
}

int map::loadSky(char *ruta){
	if (!fondo.loadSky(ruta)) return 0;	
	return 1;
}

int map::loadSound(char *ruta){
	if (SOUND_ENABLED){
		FILE* archivo;
		char tempo[30];
		int hab;
		int i,j,k;
		bool newSound=true;

		walkSound=new int[numHabitaciones];
		runSound=new int[numHabitaciones];
	
		chdir(getDirs(ruta));
		//Abrimos el archivo
		archivo=fopen (ruta,"r");

	
		chdir("../..");

		if (!archivo) return 0;

		fscanf(archivo,"%s",tempo);
		fscanf(archivo,"%s",tempo);
		
		soundRef=(int *)realloc(soundRef,(sound->getNumBuffers()+1)*sizeof(int *));
		
		chdir(getDirs(tempo));
		soundRef[0]=sound->add(tempo);
		chdir("../..");

		ambientSound=sound->sid_new(soundRef[0]);
		sound->setListener(0,0,0);
		sound->sid_pos(ambientSound,0,0,0);
		sound->sid_play(ambientSound);
		sound->sid_loop(ambientSound,true);



		for ( i=0;i<numHabitaciones && !feof(archivo);i++){
			fscanf(archivo,"%s %d",tempo,&hab);
			for (j=0;j<2 && !feof(archivo);j++){
				fscanf(archivo,"%s",tempo);
				fscanf(archivo,"%s",tempo);
				for (k=0;k<sound->getNumBuffers();k++){
					if (!strcmp(tempo,sound->getFileName(k))){
						newSound=false;
						break;
					}
				}
				if (newSound){
					soundRef=(int *)realloc(soundRef,(sound->getNumBuffers()+1)*sizeof(int *));

					chdir(getDirs(tempo));
					soundRef[k]=sound->add(tempo);
					
					if (!j) walkSound[i]=sound->sid_new(soundRef[k-1]);
					else runSound[i]=sound->sid_new(soundRef[k-1]);

					chdir("../..");
				}
				else {
					if (!j) walkSound[i]=sound->sid_new(soundRef[k]);
					else runSound[i]=sound->sid_new(soundRef[k]);
					newSound=true;
				}
				


			}
		}

		fclose(archivo);
	}
	return 1;
}

int map::playSound(int hab, int mode, GL3f pos){
	if (SOUND_ENABLED){
		if (hab<0 || hab >=numHabitaciones) return 0;
		sound->setListener(pos.x,pos.y,pos.z);
		sound->sid_pos(ambientSound,pos.x,pos.y,pos.z);
		if (mode==RUNNING_FORWARD){
			sound->sid_pos(runSound[hab],pos.x,pos.y,pos.z);
			if (sound->sid_state(runSound[hab])!=AL_PLAYING){
				for (int i=1;i<sound->getNumSources();i++){
					if (sound->sid_state(i)==AL_PLAYING)  sound->sid_stop(i);
				}
				sound->sid_play(runSound[hab]);
	
			}
		}
		else if (mode==WALKING_FORWARD){
			sound->sid_pos(walkSound[hab],pos.x,pos.y,pos.z);
			if (sound->sid_state(walkSound[hab])!=AL_PLAYING){
				for (int i=1;i<sound->getNumSources();i++){
					if (sound->sid_state(i)==AL_PLAYING)  sound->sid_stop(i);
				} 
				sound->sid_play(walkSound[hab]);
	
			}
		}
		else return 0;
	}
	return 1;
}

void map::stopSound(){
	if (SOUND_ENABLED){
		for (int i=1;i<sound->getNumSources();i++){
			if (sound->sid_state(i)==AL_PLAYING)  sound->sid_stop(i);
		}
	}
}

