#include "mates.h"

#define EPSILON 0.000001
#define CROSS(dest,v1,v2) \
          dest[0]=v1[1]*v2[2]-v1[2]*v2[1]; \
          dest[1]=v1[2]*v2[0]-v1[0]*v2[2]; \
          dest[2]=v1[0]*v2[1]-v1[1]*v2[0];
#define DOT(v1,v2) (v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2])
#define SUB(dest,v1,v2) \
          dest[0]=v1[0]-v2[0]; \
          dest[1]=v1[1]-v2[1]; \
          dest[2]=v1[2]-v2[2]; 


												
GL3f Cross(GL3f vVector1, GL3f vVector2)
{
	GL3f vNormal;	

	
	
	vNormal.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y));
														
	
	vNormal.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z));
														

	vNormal.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x));

	return vNormal;
}


GL3f Vector(GL3f vPoint1, GL3f vPoint2)
{
	GL3f vVector;			

	


	vVector.x = vPoint1.x - vPoint2.x;
	vVector.y = vPoint1.y - vPoint2.y;
	vVector.z = vPoint1.z - vPoint2.z;

	return vVector;					
}

float Magnitude(GL3f vNormal)
{


	return (float)sqrt( (vNormal.x * vNormal.x) + 
						(vNormal.y * vNormal.y) + 
						(vNormal.z * vNormal.z) );
}


GL3f Normalize(GL3f vNormal)
{
	float magnitude = Magnitude(vNormal);
	
	vNormal.x /= magnitude;	
	vNormal.y /= magnitude;	
	vNormal.z /= magnitude;	



	return vNormal;		
}


GL3f Normal(GL3f vTriangle[])					
{												
	GL3f vVector1 = Vector(vTriangle[2], vTriangle[0]);
	GL3f vVector2 = Vector(vTriangle[1], vTriangle[0]);

	//GL3f vNormal = Cross(vVector1, vVector2);
	GL3f vNormal = Cross(vVector2, vVector1);

	

	vNormal = Normalize(vNormal);			

	return vNormal;							
}

float PlaneDistance(GL3f Normal, GL3f Point)
{	
	float distance = 0;									


	distance = - ((Normal.x * Point.x) + (Normal.y * Point.y) + (Normal.z * Point.z));

	return distance;									
}

bool IntersectedPlane(GL3f vTriangle[], GL3f vLine[], float &distance,GL3f &vNormal)
{
	float distance1=0, distance2=0;				
			
	vNormal = Normal(vTriangle);
	
    float originDistance = PlaneDistance(vNormal, vTriangle[0]);

	distance1 = ((vNormal.x * vLine[0].x)  +					// Ax +
		         (vNormal.y * vLine[0].y)  +					// Bx +
				 (vNormal.z * vLine[0].z)) + originDistance;	// Cz + D


	distance2 = ((vNormal.x * vLine[1].x)  +					// Ax +
		         (vNormal.y * vLine[1].y)  +					// Bx +
				 (vNormal.z * vLine[1].z)) + originDistance;	// Cz + D



	if(distance1 * distance2 >= 0)			
	   return false;						
					
	return true;							
}


float Dot(GL3f vVector1, GL3f vVector2) 
{
	return ( (vVector1.x * vVector2.x) + (vVector1.y * vVector2.y) + (vVector1.z * vVector2.z) );
}
											
GL3f IntersectionPoint(GL3f vNormal, GL3f vLine[], float distance)
{
	GL3f vPoint , vLineDir ;	
	double Numerator = 0.0, Denominator = 0.0, dist = 0.0;

		
	vLineDir = Vector(vLine[1], vLine[0]);	
	vLineDir = Normalize(vLineDir);	
//	vNormal = Normalize(vNormal);


	Numerator = - (vNormal.x * vLine[0].x +		
			   vNormal.y * vLine[0].y +
			   vNormal.z * vLine[0].z + distance);

	Denominator = Dot(vNormal, vLineDir);	

	if( Denominator == 0.0)					
		return vLine[0];					

	dist = Numerator / Denominator;			

	vPoint.x = (float)(vLine[0].x + (vLineDir.x * dist));
	vPoint.y = (float)(vLine[0].y + (vLineDir.y * dist));
	vPoint.z = (float)(vLine[0].z + (vLineDir.z * dist));

	return vPoint;							
}


bool InsidePolygon(GL3f vIntersection, GL3f Poly[], long verticeCount)
{
	const double MATCH_FACTOR = 0.9998;		
	double Angle = 0.0;						
	GL3f vA, vB;					
	

	for (int i = 0; i < verticeCount; i++)	
	{	
		vA = Vector(Poly[i], vIntersection);
		vB = Vector(Poly[(i + 1) % verticeCount], vIntersection);

 		vA = Normalize(vA);
		vB = Normalize(vB);
												
		Angle += AngleBetweenVectors(vA, vB);
	/*	printf("->%d\n",Angle);
		printf("vA (%d): ",i); vA.print();
		printf("vB (%d): ",i); vB.print();
	*/}


//	printf ("Angle %e MATCH_FACTOR: %e Diference: %e\n",Angle,(MATCH_FACTOR * (2.0 * PI)),Angle - (MATCH_FACTOR * (2.0 * PI)));
	if(Angle >= (MATCH_FACTOR * (2.0 * PI)) )	
		return true;							
		
	return false;								
}


bool InsideTriangle(GL3f vIntersection, GL3f Poly[]) {
	double Angle = 0.0;
	GL3f vA, vB;
	double epsilon = 0.0001;

	for ( int i = 0; i < 3; i++ ) {
		vA = Vector(Poly[i], vIntersection);
		vB = Vector (Poly[(i+1)%3], vIntersection);
		vA = Normalize(vA);
		vB = Normalize(vB);

		Angle += AngleBetweenVectors(vA, vB);
	}
	//printf ("Angle: %e", Angle);
	double volta = PI*2.0;
	if ( (Angle <= (volta+epsilon)) && (Angle >= (volta-epsilon)) ) {
		return true;
	}
	return false;
}

int intersect_triangle(double orig[3], double dir[3],
                   double vert0[3], double vert1[3], double vert2[3],
                   double *t, double *u, double *v)
{
   double edge1[3], edge2[3], tvec[3], pvec[3], qvec[3];
   double det,inv_det;

   SUB(edge1, vert1, vert0);
   SUB(edge2, vert2, vert0);

   CROSS(pvec, dir, edge2);

   det = DOT(edge1, pvec);

#ifdef TEST_CULL           
   if (det < EPSILON)
      return 0;

   SUB(tvec, orig, vert0);

   *u = DOT(tvec, pvec);
   if (*u < 0.0 || *u > det)
      return 0;

   CROSS(qvec, tvec, edge1);

   *v = DOT(dir, qvec);
   if (*v < 0.0 || *u + *v > det)
      return 0;

   *t = DOT(edge2, qvec);
   inv_det = 1.0 / det;
   *t *= inv_det;
   *u *= inv_det;
   *v *= inv_det;
#else
   if (det > -EPSILON && det < EPSILON)
     return 0;
   inv_det = 1.0 / det;

   SUB(tvec, orig, vert0);

   *u = DOT(tvec, pvec) * inv_det;
   if (*u < 0.0 || *u > 1.0)
     return 0;

   CROSS(qvec, tvec, edge1);

   *v = DOT(dir, qvec) * inv_det;
   if (*v < 0.0 || *u + *v > 1.0)
     return 0;

   *t = DOT(edge2, qvec) * inv_det;
#endif
   return 1;

}


double AngleBetweenVectors(GL3f Vector1, GL3f Vector2)
{
	float dotProduct = Dot(Vector1, Vector2);				


	float vectorsMagnitude = Magnitude(Vector1) * Magnitude(Vector2) ;


	double angle = acos( dotProduct / vectorsMagnitude );

	if(isnan(angle))
		return 0;
	
	return( angle );
}

bool pointInsideLine(GL3f point, GL3f line[2] ) {

	float alfa;
	GL3f vecDir = line[1] - line[0];
	alfa = ( point.x - line[0].x ) / Magnitude (vecDir);
	printf("%f\n",alfa);
	if ( ( alfa >= 0 ) && ( alfa <= 1 ) ) {
		return true;
	}
	return false;

}


double testline(GL3f vLine[2], GL3f &pres, GL3f vPoly[3])
{

	GL3f v1 = vPoly[1] - vPoly[0];
	GL3f v2 = vPoly[2] - vPoly[0];

	GL3f r, vec;
	r = vLine[0];
	vec = vLine[1] - vLine[0];
	
	GL3f n;

	n = Cross(v1,v2);
	n = Normalize(n);
		
		

	float fres;
	double t;
	float d = -1.0f * (n.x * vPoly[0].x + n.y * vPoly[0].y + n.z * vPoly[0].z);
		
	fres=vec*n;
	pres=r;
	//printf("->%f %f %f\n",r.x,r.y,r.z);
//	if ((fres)!=0)
	if ( (fres < -0.000000001) || (fres > 0.000000001))
		{
		t=-(n*r + d)/fres;
		
		}
	// hack... arreglar
	else t=0.0;
    return t;

	
	

}
float toRadiants(float num){
	return num*(float)PI/180.0f;
}

float toDegrees(float num){
	return num*180.0f/(float)PI;
}
