#include "render.h"

extern SDL_Surface *WINDOW;

render::render()
{
	delay=1000;
	nextChange=0;
	night=GL3f(1.0f,1.0f,1.0f);

	const SDL_VideoInfo* info = NULL;
    
	//Here we read the video configuration file
	FILE *fd;

	fd=fopen("config.vid","r");
	if (!fd) 
	{
		printf("config.vid not found...loading defaults.\n");
		TRANSPARENCIA=1;
		BPP=32;
		FULLSCREEN=0;
		WIDTH=640;
		HEIGHT=480;
	} else {
	char buf[100];
	
	while(!feof(fd))
		{
		buf[0]=0;
		fscanf(fd,"%s",buf);
		if (!strcmp("TRANS",buf)) fscanf(fd,"%d",&TRANSPARENCIA);
		else if (!strcmp("BPP",buf)) fscanf(fd,"%d",&BPP);
		else if (!strcmp("FULLSCREEN",buf)) fscanf(fd,"%d",&FULLSCREEN);
		else if (!strcmp("RES",buf)) fscanf(fd,"%d %d",&WIDTH,&HEIGHT);
		}
	
	fclose (fd);
	}

	printf("[i] Initializing SDL\n");

	//Here we init SDL
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
	{
        	fprintf(stderr, "Cannot init SDL (%s)\n", SDL_GetError());
	        killProgram (0);
	}
	
	info = SDL_GetVideoInfo( );

	if (FULLSCREEN)FULLSCREEN=SDL_FULLSCREEN;
	WINDOW = SDL_SetVideoMode (  WIDTH,HEIGHT, BPP,
				     SDL_OPENGL|SDL_HWSURFACE|SDL_DOUBLEBUF|
				     FULLSCREEN);
	if(! WINDOW )
	{
		fprintf(stderr,"ERROR setting video mode.\n");
		killProgram (0);
	}
	SDL_ShowCursor(false);
	SDL_EnableKeyRepeat(1, 0);
	SDL_WM_SetCaption("GLH","");
	
	// glEnable
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	// glTextures
	glTexEnvi(GL_TEXTURE,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	// glHint
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glHint(GL_POLYGON_SMOOTH_HINT,  GL_NICEST);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

	SDL_GL_SwapBuffers( );
};

void render::setTexture(texture textureObj){

	if (textureObj.bpp==32 && !glIsEnabled(GL_ALPHA_TEST) && TRANSPARENCIA){
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER,0.9f);
	}
	else if (textureObj.bpp==24 && glIsEnabled(GL_ALPHA_TEST)){
		glDisable(GL_ALPHA_TEST);
	}

	//Warrada para solucionar un problema al cargar texturas
	if (textureObj.idTex>999) glBindTexture(GL_TEXTURE_2D, 0);
	else glBindTexture(GL_TEXTURE_2D, textureObj.idTex);
}

void render::dibuja(stage *fase)
{
	//Limpiamos la pantalla
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

	int *PVS=0;

	//Cojemos el PVS de la habitacion en la que estamos
#if _PVS
	PVS=fase->getPVS(fase->getCamaraId());
#endif

	//Reseteamos el glLineWidth a 2
	glLineWidth(2.0f);

	//Miramos que en que tipo de debug estamos
	switch(DEBUG){
	case SOLO_CARAS:
		glPolygonMode(GL_FRONT, GL_FILL);
		modo=1;
		break;
	case SOLO_VERTICES:
		glPolygonMode(GL_FRONT, GL_LINE);
		modo=1;
		break;
	case CARAS_Y_VERTICES:
		glPolygonMode(GL_FRONT, GL_LINE);
		modo=2;
		break;
	}

	//Activamos la textura o no dependiendo del modo en que estamos
	if (TEXTURA && modo==1) glEnable(GL_TEXTURE_2D);
	else glDisable(GL_TEXTURE_2D);

	//Nos colocamos en el mundo y empezamos a mirar
	pintaCamara(fase);

	for (;modo>0;modo--){
		pintaEscenario(fase,PVS);
		pintaItems(fase,PVS);
		pintaEnemigos(fase,PVS);
			
		//Modo 2 se usa en modo CARAS_Y_VERTICES.
		//Cuando esta activado pinta las linias que forman los triangulos
		//Una vez hecho pasa a modo 1 y pinta normal
		if (modo==2) {
			glPolygonMode(GL_FRONT, GL_FILL);
			if (TEXTURA) glEnable(GL_TEXTURE_2D);
		}
	};

	if (BOUNDING) pintaBoundings(fase,PVS);
#if _PVS
	if (PORTALES) pintaPortales(fase,PVS);
#endif
	drawSkyBox(fase->getMap()->getSky(),fase->getCamaraPos());
	fase->getMessageControl()->drawMessages();
	fase->getCamaraInterface()->paint();
#if _PVS
	delete[]PVS;
#endif
}

void render::pintaCamara(stage *fase){

	GL3f pos=fase->getCamaraPos();
	GL3f obj=fase->getCamaraObj();

	//printf("my Poss:");pos.print();

#if SOUND
	fase->getMap()->playSound(fase->getCamaraId(),RUNNING,pos);
#endif

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0,WIDTH/HEIGHT,0.1,3000.0);  
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(pos.x,pos.y+PLAYER_HEIGHT,pos.z,obj.x,obj.y+PLAYER_HEIGHT,obj.z,0.0f,1.0f,0.0f);
			
	glPushMatrix();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	control *camara=fase->getCamFirstView();
	
	 
	glTranslatef(0.0f,-0.05f,-0.1f);


	drawObject (	camara->getObject3d()->getTexture(),
			camara->getObject3d()->getVertex(camara->getPosAnimacio()),
			camara->getObject3d()->getTextureVertex(),
			camara->getObject3d()->getNumFaces());
	glPopMatrix();
}

void render::pintaEscenario(stage *fase, int *PVS){
	//Miramos todas las habitaciones
	for (int cont=0;cont<fase->getMap()->getNumHabitaciones();cont++)
	{
		habitacion *hab=fase->getMap()->getHabitacion(cont);
		//Si desde donde estamos, vemos esa habitacion la pintamos con vertexArray
#if _PVS
		if(PVS[hab->getId()]){
#endif
			beginVertexArray(hab->getVertex(),hab->getTextureVertex());
						
			for (int i=0; i<hab->getNumTextures();i++)
			{
			drawVertexArray( fase->getMap()->getTexture(hab->getTextureNumber(i)),
					 hab->getFirstTexture(i)/3, 
					 hab->getFirstTexture(i+1)/3 );
			}
			endVertexArray();
		}
	}
}

void render::pintaItems(stage *fase, int *PVS)
{
	register unsigned int i=0;
	for (i=0;i<fase->getNumItems();i++)
	{
		control *item=fase->getItem(i);
		//Miramos si vemos ese item desde nuestra habitacion
#if _PVS
		if (PVS[item->getId()]) {
#else
		if (item->isVisible()) 
		{
#endif
			//Cojemos el frame actual de la animacion
			int frame=item->getPosAnimacio();

			//Cambiamos la textura
		 	setTexture(item->getObject3d()->getTexture());

			GL3f positionObj=item->getPos();
			glPushMatrix();

			//Colocamos el item en la posicion y rotacion que le corresponde
			glhTranslatef(positionObj);
			glRotatef(item->getRotation(),0,1,0);
			//Lo pintamos
			
			if (item->isExploting())
			{
			drawObject(     item->getObject3d()->getTexture(),
					item->getExplotingVertex(),
					item->getObject3d()->getTextureVertex(),
					item->getObject3d()->getNumFaces() );
			} else {
			if (item->isDesintegrating())
			{
			drawObject(	item->getObject3d()->getTexture(),
					item->getObject3d()->getVertex(frame),
					item->getObject3d()->getTextureVertex(),
					item->getNumDFaces() );
			drawObject(	item->getObject3d()->getTexture(),
					item->getObject3d()->getVertex(frame),
					item->getObject3d()->getTextureVertex(),
					item->getObject3d()->getNumFaces(),0.5f);
			} else {
			drawObject(	item->getObject3d()->getTexture(),
					item->getObject3d()->getVertex(frame),
					item->getObject3d()->getTextureVertex(),
					item->getObject3d()->getNumFaces() );
			}
			glPopMatrix();
		}
	}
}
}


void render::pintaEnemigos(stage *fase, int *PVS)
{
	register unsigned int i=0;
	for (int i=0;i<fase->getMaxPlayers();i++){
		enemy *enemigo=fase->getEnemy(i);
		
		if (enemigo){
		//Miramos si vemos el enemigo desde la habitacion donde estamos
#if _PVS
			if (PVS[enemigo->getId()] && enemigo->isVisible()){
#else
			if (enemigo->isVisible()){
#endif
				glPushMatrix();

	//glRotatef(enemigo->getRotation(),0,1,0);			
				//Cojemos cada una de las 3 partes que lo forman
				for (int j=0;j<3;j++){
			
					//Miramos en que frame esta actualmente la parte del enemigo
					int frame=enemigo->getControl(j)->getPosAnimacio();

					//Le aplicamos la textura que necesite
					GL3f positionObj=enemigo->getControl(j)->getPos();

					//Lo colocamos en el lugar que haga falta
					glhTranslatef(enemigo->getUnion(j));
					if (!j)	glRotatef(-enemigo->getRotation()-90,0,1,0);
						
					//Lo pintamos
					if (enemigo->isDying()){
						drawObject(	enemigo->getControl(j)->getObject3d()->getTexture(),
								enemigo->getControl(j)->getObject3d()->getVertex(frame),
								enemigo->getControl(j)->getObject3d()->getTextureVertex(),
								enemigo->getControl(j)->getNumDFaces(),0.5f);

						drawObject(	enemigo->getControl(j)->getObject3d()->getTexture(),
								enemigo->getControl(j)->getExplotingVertex(),
								enemigo->getControl(j)->getObject3d()->getTextureVertex(),
								enemigo->getControl(j)->getObject3d()->getNumFaces());
					}
					else {
						//Esto es para solucionar los problemas d la textura
						texture enemigoT;
						switch(j)
						{
						case HEAD: enemigoT.idTex=idHead;break;
						case BODY: enemigoT.idTex=idBody;break;
						case LEGS: enemigoT.idTex=idLegs;break;
						}
							
						/*drawObject(enemigo->getControl(cont3)->getObject3d()->getTexture(),enemigo->getControl(cont3)->getObject3d()->getVertex(frame),
							enemigo->getControl(cont3)->getObject3d()->getTextureVertex(),enemigo->getControl(cont3)->getNumDFaces());
		*/
						drawObject(	enemigoT,enemigo->getControl(j)->getObject3d()->getVertex(frame),
								enemigo->getControl(j)->getObject3d()->getTextureVertex(),
								enemigo->getControl(j)->getNumDFaces());
					}
				}
				glPopMatrix();

				glPushMatrix();
				int frame=enemigo->getControl(BODY)->getPosAnimacio();
				int weaponFrame=enemigo->getWeapon()->getThirdView()->getPosAnimacio();
				//Le aplicamos la textura que necesite
				glhTranslatef(enemigo->getUnion(LEGS));
				glRotatef(-enemigo->getRotation()-90,0,1,0);
				glhTranslatef(enemigo->getUnion(BODY));
				glhTranslatef(enemigo->getUnion(WEAPON_POS));
				
				//glRotatef(enemigo->getWeaponRot().x,0,1,0);
				glRotatef(enemigo->getWeaponRot().y,1,0,0);
				//printf("%f %f\n",enemigo->getWeaponRot().x,enemigo->getWeaponRot().y);
				//	enemigo->getUnion(WEAPON_POS).print();
				drawObject(	enemigo->getWeapon()->getThirdView()->getObject3d()->getTexture(),
						enemigo->getWeapon()->getThirdView()->getObject3d()->getVertex(weaponFrame),
						enemigo->getWeapon()->getThirdView()->getObject3d()->getTextureVertex(),
						enemigo->getWeapon()->getThirdView()->getNumDFaces());
				glPopMatrix();
			}
		}
	}

}

#if _PVS
void render::pintaPortales(stage *fase, int *PVS){
	//Puntero temporal
	GL3f *vertex;

	//Si solo pintamos vertices hacemos las linias mas gruesas para que se vean mas 
	if (DEBUG==SOLO_VERTICES) glLineWidth(10.0f);

	//Si no pintamos texturas, las activamos para hacer las transparencias correctamente
	if (!TEXTURA) glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);
	glColor4f(0.0f,1.0f,1.0f,0.5f);
	glBegin(GL_QUADS);
//	printf("habitacion: %d numPortals: %d\n",fase->getCamaraId(),fase->getMap()->getHabitacion(fase->getCamaraId())->getNumPortals());
	//Cojemos todos los portales de nuestra habitacion y los pintamos
	for (int cont=0; cont<fase->getMap()->getHabitacion(fase->getCamaraId())->getNumPortals();cont++){
		vertex=fase->getMap()->getHabitacion(fase->getCamaraId())->getPortalVertex(cont);
						
		glhVertex3f(vertex[1]);
		glhVertex3f(vertex[3]);
		glhVertex3f(vertex[2]);
		glhVertex3f(vertex[0]);				
	}

	glPopMatrix();
	glEnd();
	glDisable(GL_BLEND);

	//Si no pintamos texturas las deshabilitamos
	if (!TEXTURA) glDisable(GL_TEXTURE_2D);

	//Reseteamos el grosor de la linia a 2
	if (DEBUG==SOLO_VERTICES) glLineWidth(2.0f);

		
}
#endif

void render::pintaBoundings(stage *fase, int *PVS){
	int cont=0;

	//Pintamos el bounding de los enemigos
	for (cont=0;cont<fase->getActualPlayers();cont++){
		enemy *enemigo=fase->getEnemy(cont);
#if _PVS
		if (PVS[enemigo->getId()]){
#endif
			glPushMatrix();
			glhTranslatef(enemigo->getPos());
			drawBoundingCube(enemigo->getBounding());			
			glPopMatrix();
#if _PVS
		}
#endif
			
	}

	//Pintamos el bounding de los items
	for (cont=0;cont<fase->getNumItems();cont++){
		control *item_tempo=fase->getItem(cont);
#if _PVS
		if (PVS[item_tempo->getId()]){
#endif
			glPushMatrix();
			glhTranslatef(item_tempo->getPos());
			drawBoundingCube(item_tempo->getObject3d()->getBounding());
			glPopMatrix();
#if _PVS
		}
#endif
	}

}

void render::beginVertexArray(float *vertexArray, float *textureArray){

	//Habilitamos el vertexArray
	glEnableClientState(GL_VERTEX_ARRAY); 
	glVertexPointer(3,GL_FLOAT,0,vertexArray); 

	//Si pintamos texturas le pasamos el array de texturas al glTexCoordPointer.
	//Si no las pintamos, le pasamos el array de vertices al glColorPointer para conseguir
	//un colorido paranoico
	if (TEXTURA && modo!=2){
		glEnableClientState(GL_TEXTURE_COORD_ARRAY); 
		glTexCoordPointer(2,GL_FLOAT,0,textureArray); 
	}
	else if (!TEXTURA && modo!=2) {
		glEnableClientState(GL_COLOR_ARRAY);
		glColorPointer(3,GL_FLOAT,0,vertexArray); 
	}

}

void render::drawVertexArray(texture textura, int begin, int end, float A, float R, float G, float B){

	if (A<0) glColor3f(R, G, B);
	else {
		glEnable(GL_BLEND);
		glBlendFunc(GL_ONE, GL_ONE);
		glColor4f (R, G, B, A);
	}



	//Si necesitas la textura la carga
	if (TEXTURA && modo!=2) setTexture(textura);
	
	//Pintamos el mapa
	glDrawArrays(GL_TRIANGLES,begin*3,(end-begin)*3); 

	if (A>=0) glDisable(GL_BLEND);
}

void render::endVertexArray(){

	//Desactivamos el vertexArray
	glDisableClientState(GL_VERTEX_ARRAY);

	//Dependiendo del modo que utilizemos desactivamos el GL_TEXTURE_COORD_ARRAY o el
	//GL_COLOR_ARRAY
	if (TEXTURA && modo!=2) glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	else if (!TEXTURA  && modo!=2) glDisableClientState(GL_COLOR_ARRAY);
}


void render::drawObject(texture textura, float *vertex, float *textureVertex, int numFaces, float A, float R, float G, float B){
	
	if (A<0) glColor3f(R, G, B);
	else {
		glEnable(GL_BLEND);
		glBlendFunc(GL_ONE, GL_ONE);
		glColor4f (R, G, B, A);
	}

	//Si usamos texturas las activamos y las pintamos.
	//Si no las usamos no las pintamos
	if (TEXTURA && modo!=2){
		setTexture(textura);
		glBegin(GL_TRIANGLES);

		for (register int i=0,j=0;i<numFaces*9;i+=3, j+=2){
			glTexCoord2f(textureVertex[j],textureVertex[j+1]);
			glVertex3f(vertex[i],vertex[i+1],vertex[i+2]);
		}

		glEnd();
	}else{
		glBegin(GL_TRIANGLES);

		if (!TEXTURA && modo!=2) glColor3f(1.0f,0.0f,0.0f);
		for (register int i=0;i<numFaces*9;i+=3){
			glVertex3f(vertex[i],vertex[i+1],vertex[i+2]);
		}

		glEnd();
	}

	if (A>=0) glDisable(GL_BLEND);

}

void render::drawBoundingCube(GLBounding bounding){

	if (!TEXTURA) glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);

	//Si no pintamos solamente las linias que forman los triangulos desactivamos el GL_CULL_FACE
	//porque si no hace un efecto malo.
	//En caso contrario aumentamos la anchura de la linia
	if (DEBUG!=SOLO_VERTICES) 	glDisable(GL_CULL_FACE);
	else glLineWidth(5.0f);

	
	glColor4f(1.0f,1.0f,0.0f,1.0f);

	//Pintamos el bounding	
	glBegin(GL_QUADS);
		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmin());
		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmin());

		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmax());
		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmax());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmax());
		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmax());

		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmin());
		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmax());
		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmax());

		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmax());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmax());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmin());

		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmin(),bounding.getZmax());
		glVertex3f(bounding.getXmin(),bounding.getYmin(),bounding.getZmax());

		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmin());
		glVertex3f(bounding.getXmax(),bounding.getYmax(),bounding.getZmax());
		glVertex3f(bounding.getXmin(),bounding.getYmax(),bounding.getZmax());

	glEnd();

	//Lo dejamos todo como antes
	if (DEBUG!=SOLO_VERTICES)	glEnable(GL_CULL_FACE);
	else glLineWidth(2.0f);
	
	glDisable(GL_BLEND);
	if (!TEXTURA) glDisable(GL_TEXTURE_2D);
}

void render::drawSkyBox(Sky cielo,GL3f pos)
{
	
	cielo.x = cielo.x - cielo.width  / 2;
	cielo.y = cielo.y - cielo.height / 2;
	cielo.z = cielo.z - cielo.length / 2;

	glPushMatrix();
	glhTranslatef(pos);

	
	if (SDL_GetTicks()>=nextChange){
		nextChange=SDL_GetTicks()+delay;
		tm *timeInfo;
		time_t rawtime;
		time(&rawtime);
		timeInfo=localtime(&rawtime);
		int hour=timeInfo->tm_hour;

		if (hour>=23  || (hour>=0 && hour<=5)) night=GL3f(0.4f,0.3f,0.3f);
		else if (hour==6) night=GL3f(0.7f,0.5f,0.5f);
		else if (hour==7) night=GL3f(0.9f,0.6f,0.6f);
		else if (hour==8) night=GL3f(1.0f,0.8f,0.8f);
		else if (hour>=9 && hour<=11) night=GL3f(1.0f,0.8f,0.8f);
		else if (hour>=12 && hour<=19) night=GL3f(1.0f,1.0f,1.0f);
		else if (hour==20) night=GL3f(0.9f,0.8f,0.8f);
		else if (hour==21) night=GL3f(0.8f,0.5f,0.5f);
		else if (hour==22) night=GL3f(0.5f,0.4f,0.4f);
	}

	
	glColor3f(night.x,night.y,night.z);
	setTexture(cielo.g_Texture[BACK_ID]);
	glBegin(GL_QUADS);		
		
		glTexCoord2f(1.0f, 0.0f); glVertex3f(cielo.x + cielo.width, cielo.y,			cielo.z);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(cielo.x + cielo.width, cielo.y + cielo.height, cielo.z); 
		glTexCoord2f(0.0f, 1.0f); glVertex3f(cielo.x ,			cielo.y + cielo.height, cielo.z);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(cielo.x,			cielo.y,			cielo.z);
		
	glEnd();

	// Bind the FRONT texture of the sky map to the FRONT side of the box

	setTexture(cielo.g_Texture[FRONT_ID]);
	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);	
	
		// Assign the texture coordinates and vertices for the FRONT Side
		glTexCoord2f(1.0f, 0.0f); glVertex3f(cielo.x ,		cielo.y,			cielo.z + cielo.length);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(cielo.x,			cielo.y + cielo.height, cielo.z + cielo.length);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(cielo.x + cielo.width , cielo.y + cielo.height, cielo.z + cielo.length); 
		glTexCoord2f(0.0f, 0.0f); glVertex3f(cielo.x + cielo.width , cielo.y,			cielo.z + cielo.length);
	glEnd();

	// Bind the BOTTOM texture of the sky map to the BOTTOM side of the box
	setTexture(cielo.g_Texture[BOTTOM_ID]);
	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);		
	
		// Assign the texture coordinates and vertices for the BOTTOM Side
		glTexCoord2f(1.0f, 0.0f); glVertex3f(cielo.x,			cielo.y,			cielo.z);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(cielo.x,			cielo.y,			cielo.z + cielo.length);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(cielo.x + cielo.width, cielo.y,			cielo.z + cielo.length); 
		glTexCoord2f(0.0f, 0.0f); glVertex3f(cielo.x + cielo.width, cielo.y,			cielo.z);
	glEnd();

	// Bind the TOP texture of the sky map to the TOP side of the box
	setTexture(cielo.g_Texture[TOP_ID]);
	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);		
		
		// Assign the texture coordinates and vertices for the TOP Side
		glTexCoord2f(0.0f, 1.0f); glVertex3f(cielo.x + cielo.width, cielo.y + cielo.height, cielo.z);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(cielo.x + cielo.width, cielo.y + cielo.height, cielo.z + cielo.length); 
		glTexCoord2f(1.0f, 0.0f); glVertex3f(cielo.x,			cielo.y + cielo.height,	cielo.z + cielo.length);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(cielo.x,			cielo.y + cielo.height,	cielo.z);
		
	glEnd();

	// Bind the LEFT texture of the sky map to the LEFT side of the box
	setTexture(cielo.g_Texture[LEFT_ID]);
	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);		
		
		// Assign the texture coordinates and vertices for the LEFT Side
		glTexCoord2f(1.0f, 1.0f); glVertex3f(cielo.x,			cielo.y + cielo.height,	cielo.z);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f(cielo.x,			cielo.y + cielo.height,	cielo.z + cielo.length ); 
		glTexCoord2f(0.0f, 0.0f); glVertex3f(cielo.x,			cielo.y,			cielo.z + cielo.length );
		glTexCoord2f(1.0f, 0.0f); glVertex3f(cielo.x,			cielo.y,			cielo.z );		
		
	glEnd();

	// Bind the RIGHT texture of the sky map to the RIGHT side of the box
	setTexture(cielo.g_Texture[RIGHT_ID]);
	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);		

		// Assign the texture coordinates and vertices for the RIGHT Side
		glTexCoord2f(0.0f, 0.0f); glVertex3f(cielo.x + cielo.width, cielo.y,			cielo.z );
		glTexCoord2f(1.0f, 0.0f); glVertex3f(cielo.x + cielo.width, cielo.y,			cielo.z + cielo.length );
		glTexCoord2f(1.0f, 1.0f); glVertex3f(cielo.x + cielo.width, cielo.y + cielo.height,	cielo.z + cielo.length ); 
		glTexCoord2f(0.0f, 1.0f); glVertex3f(cielo.x + cielo.width, cielo.y + cielo.height,	cielo.z);
	glEnd();

  glPopMatrix();
  }


