
#ifndef _STAGE_H_
#define _STAGE_H_

//#include "collision.h"
#include "structures.h"
#include "item.h"
#include "help.h"
#include "camara.h"
#include "enemy.h"
#include "mates.h"
#include "messageControl.h"
#include "map.h"
#include "weapon.h"

extern void killProgram(int mode);

class stage{

	//Controls all the messages painted on the screen
	messageControl *mensajes;

#if _PVS
	//Here is where the PVS is loaded
	int *PVS;
#endif

	//Array to load the items
	control **itemObj;

	//Array with the different item models
	object3d **itemModel;

	//Array with all the weapons
	weapon **weaponObj;

	//Number of weapons
	int numWeapons;

	//Your player
	camara *camaraObj;

	//The enemies
	enemy **enemigos;

	//Here is where all the static map info is loaded (rooms, portals, etc).
	map *mapa;

	//Number of items
	int numItems;

	//Number of item models
	int numItemModels;

	//Length of the players array
	int maxPlayers;

	//Number of actual players
	int actualPlayers;

			
private:
	
	//Reads the map file. Returns 1 if everything it's ok. Returns 0 if an error happens.
	int leeFichero(char *ruta);

	//Checks in which room you are. You must pass it your actual position
	int checkCamera(GL3f pos);

#if _PVS
	//Loads the pvs. Waits the PVS file
	int cargaPVS(char *ruta);
#endif
	
public:
	//Inits vars and reads the file info. It waits the direction of a map file.
	stage(char *ruta);

	//Frees memory
	~stage();

	//Returns the number of items
	int getNumItems();

	//Returns the item you ask for
	control *getItem(int pos);

	//Update everything in the stage (animations, etc).
	void update();

#if _PVS
	//Returns the PVS array of the room you ask for
	int *getPVS(int pos);


	//Returns the id of where you are
	int getCamaraId();
#endif

	//Returns your position
	GL3f getCamaraPos();

	//Returns where you are looking at
	GL3f getCamaraObj();

	//Returns your bounding
	GLBounding getCamaraBounding();

	//Used to set your position. The first parameter is the position and the second one define
	//if we must check in which room we are
	void setCamaraPos(GL3f posCam, bool collision=true);

	//Changes where you are looking at
	void setCamaraObj(double yawH, double yawV);

	//Changes you hab id
	void setCamaraId(int id);


	//Returns the messageControl object
	messageControl *getMessageControl();

	//Returns the map object
	map *getMap();

	//Adds and enemy. The first parameter is the enemy caracter file, the second one is your nick
	//and the third one is the position in the array to be loaded
	bool addEnemy(char *ruta, char *nick, int pos);

	//Returns the enemy you ask for
	enemy *getEnemy(int which);

	//Returns the actual number of players
	int getActualPlayers();

	//Returns the max number of players
	int getMaxPlayers();

	//Returns your first view model
	control *getCamFirstView();

	//Returns your third view model
	control *getCamThirdView();

	//Kills the enemy you ask for
	bool killEnemy(int who);

	//Sets your identification number in the game
	void setCamaraDNI(int num);

	//Returns your identification number in the game
	int getCamaraDNI();

	//Sets your nick
	void setCamaraNick(char *nick);

	//Returns your nick name
	char *getCamaraNick();

	bool existsEnemy(int who);
	
	weapon *getCamaraWeapon();

	Interface *getCamaraInterface();
	
};	

#endif
